﻿using System.Collections.Generic;
using System.ComponentModel;
using Loki.Bot.Settings;
using Loki.Utilities;
using Newtonsoft.Json;

namespace FlickerStrike
{
    /// <summary>Settings for the FlickerStrike. </summary>
    public class FlickerStrikeSettings : JsonSettings
    {
        private static FlickerStrikeSettings _instance;

        /// <summary>The current instance for this class. </summary>
        public static FlickerStrikeSettings Instance
        {
            get { return _instance ?? (_instance = new FlickerStrikeSettings()); }
        }

        /// <summary>The default ctor. Will use the settings path "FlickerStrike".</summary>
        public FlickerStrikeSettings()
            : base(GetSettingsFilePath(Configuration.Instance.Name, string.Format("{0}.json", "FlickerStrike")))
        {

        }

        private int _flickerstrikeSlot;
        private int _frenzychargeSlot;
        private int _combatRange;
        private int _maxMeleeRange;
        private int _maxRangeRange;
        private bool _alwaysAttackInPlace;

        private int _moltenShellDelayMs;
        private int _flickerstrikeCooldownMs;
        private int _totemDelayMs;
        private int _trapDelayMs;
        private int _maxFlameBlastCharges;
        private int _arcticArmourCastMode;

        private int _summonSkeletonCountPerDelay;
        private int _summonSkeletonDelayMs;

        private int _summonRagingSpiritCountPerDelay;
        private int _summonRagingSpiritDelayMs;
        private int _vaalDiscTrigger;

        private int _mineDelayMs;
		
		/// <summary>
        /// How many casts to perform before the delay happens.
        /// </summary>
        [DefaultValue(15)]
        public int VaalDiscTrigger
        {
            get { return _vaalDiscTrigger; }
            set
            {
                if (value.Equals(_vaalDiscTrigger))
                {
                    return;
                }
                _vaalDiscTrigger = value;
                NotifyPropertyChanged(() => VaalDiscTrigger);
            }
        }

        /// <summary>
        /// How many casts to perform before the delay happens.
        /// </summary>
        [DefaultValue(3)]
        public int SummonRagingSpiritCountPerDelay
        {
            get { return _summonRagingSpiritCountPerDelay; }
            set
            {
                if (value.Equals(_summonRagingSpiritCountPerDelay))
                {
                    return;
                }
                _summonRagingSpiritCountPerDelay = value;
                NotifyPropertyChanged(() => SummonRagingSpiritCountPerDelay);
            }
        }

        /// <summary>
        /// How long should the CR wait after performing all the casts.
        /// </summary>
        [DefaultValue(5000)]
        public int SummonRagingSpiritDelayMs
        {
            get { return _summonRagingSpiritDelayMs; }
            set
            {
                if (value.Equals(_summonRagingSpiritDelayMs))
                {
                    return;
                }
                _summonRagingSpiritDelayMs = value;
                NotifyPropertyChanged(() => SummonRagingSpiritDelayMs);
            }
        }

        /// <summary>
        /// How many casts to perform before the delay happens.
        /// </summary>
        [DefaultValue(2)]
        public int SummonSkeletonCountPerDelay
        {
            get { return _summonSkeletonCountPerDelay; }
            set
            {
                if (value.Equals(_summonSkeletonCountPerDelay))
                {
                    return;
                }
                _summonSkeletonCountPerDelay = value;
                NotifyPropertyChanged(() => SummonSkeletonCountPerDelay);
            }
        }

        /// <summary>
        /// How long should the CR wait after performing all the casts.
        /// </summary>
        [DefaultValue(5000)]
        public int SummonSkeletonDelayMs
        {
            get { return _summonSkeletonDelayMs; }
            set
            {
                if (value.Equals(_summonSkeletonDelayMs))
                {
                    return;
                }
                _summonSkeletonDelayMs = value;
                NotifyPropertyChanged(() => SummonSkeletonDelayMs);
            }
        }

        /// <summary>
        /// How long should the CR wait before using mines again.
        /// </summary>
        [DefaultValue(5000)]
        public int MineDelayMs
        {
            get { return _mineDelayMs; }
            set
            {
                if (value.Equals(_mineDelayMs))
                {
                    return;
                }
                _mineDelayMs = value;
                NotifyPropertyChanged(() => MineDelayMs);
            }
        }

        /// <summary>
        /// Should the CR always attack in place.
        /// </summary>
        [DefaultValue(false)]
        public bool AlwaysAttackInPlace
        {
            get { return _alwaysAttackInPlace; }
            set
            {
                if (value.Equals(_alwaysAttackInPlace))
                {
                    return;
                }
                _alwaysAttackInPlace = value;
                NotifyPropertyChanged(() => AlwaysAttackInPlace);
            }
        }

        /// <summary>
        /// The skill slot to use in melee range.
        /// </summary>
        [DefaultValue(-1)]
        public int FlickerStrikeSlot
        {
            get { return _flickerstrikeSlot; }
            set
            {
                if (value.Equals(_flickerstrikeSlot))
                {
                    return;
                }
                _flickerstrikeSlot = value;
                NotifyPropertyChanged(() => FlickerStrikeSlot);
            }
        }

        /// <summary>
        /// The skill slot to use outside of melee range.
        /// </summary>
        [DefaultValue(-1)]
        public int FrenzyChargeSlot
        {
            get { return _frenzychargeSlot; }
            set
            {
                if (value.Equals(_frenzychargeSlot))
                {
                    return;
                }
                _frenzychargeSlot = value;
                NotifyPropertyChanged(() => FrenzyChargeSlot);
            }
        }

        /// <summary>
        /// Only attack mobs within this range.
        /// </summary>
        [DefaultValue(70)]
        public int CombatRange
        {
            get { return _combatRange; }
            set
            {
                if (value.Equals(_combatRange))
                {
                    return;
                }
                _combatRange = value;
                NotifyPropertyChanged(() => CombatRange);
            }
        }

        /// <summary>
        /// How close does a mob need to be to trigger the Melee skill.
        /// Do not set too high, as the cursor will overlap the GUI.
        /// </summary>
        [DefaultValue(10)]
        public int MaxMeleeRange
        {
            get { return _maxMeleeRange; }
            set
            {
                if (value.Equals(_maxMeleeRange))
                {
                    return;
                }
                _maxMeleeRange = value;
                NotifyPropertyChanged(() => MaxMeleeRange);
            }
        }

        /// <summary>
        /// How close does a mob need to be to trigger the Ranged skill.
        /// Do not set too high, as the cursor will overlap the GUI.
        /// </summary>
        [DefaultValue(35)]
        public int MaxRangeRange
        {
            get { return _maxRangeRange; }
            set
            {
                if (value.Equals(_maxRangeRange))
                {
                    return;
                }
                _maxRangeRange = value;
                NotifyPropertyChanged(() => MaxRangeRange);
            }
        }

        /// <summary>
        /// How many flameblast charges to build up before releasing.
        /// </summary>
        [DefaultValue(5)]
        public int MaxFlameBlastCharges
        {
            get { return _maxFlameBlastCharges; }
            set
            {
                if (value.Equals(_maxFlameBlastCharges))
                {
                    return;
                }
                _maxFlameBlastCharges = value;
                NotifyPropertyChanged(() => MaxFlameBlastCharges);
            }
        }

        /// <summary>
        /// The delay between casting molten shell in combat.
        /// </summary>
        [DefaultValue(5000)]
        public int MoltenShellDelayMs
        {
            get { return _moltenShellDelayMs; }
            set
            {
                if (value.Equals(_moltenShellDelayMs))
                {
                    return;
                }
                _moltenShellDelayMs = value;
                NotifyPropertyChanged(() => MoltenShellDelayMs);
            }
        }

        /// <summary>
        /// The casting mode for AA.
        /// 1 - Always cast.
        /// 2 - Only cast when monsters are in combat range.
        /// </summary>
        [DefaultValue(1)]
        public int ArcticArmourCastMode
        {
            get { return _arcticArmourCastMode; }
            set
            {
                if (value.Equals(_arcticArmourCastMode))
                {
                    return;
                }
                _arcticArmourCastMode = value;
                NotifyPropertyChanged(() => ArcticArmourCastMode);
            }
        }

        /// <summary>
        /// The delay between casting totems in combat.
        /// </summary>
        [DefaultValue(5000)]
        public int TotemDelayMs
        {
            get { return _totemDelayMs; }
            set
            {
                if (value.Equals(_totemDelayMs))
                {
                    return;
                }
                _totemDelayMs = value;
                NotifyPropertyChanged(() => TotemDelayMs);
            }
        }
		
		/// <summary>
        /// The delay between casting totems in combat.
        /// </summary>
        [DefaultValue(2000)]
        public int FlickerStrikeCooldownMs
        {
            get { return _flickerstrikeCooldownMs; }
            set
            {
                if (value.Equals(_flickerstrikeCooldownMs))
                {
                    return;
                }
                _flickerstrikeCooldownMs = value;
                NotifyPropertyChanged(() => FlickerStrikeCooldownMs);
            }
        }

        /// <summary>
        /// The delay between casting traps in combat.
        /// </summary>
        [DefaultValue(2500)]
        public int TrapDelayMs
        {
            get { return _trapDelayMs; }
            set
            {
                if (value.Equals(_trapDelayMs))
                {
                    return;
                }
                _trapDelayMs = value;
                NotifyPropertyChanged(() => TrapDelayMs);
            }
        }

        [JsonIgnore] private List<int> _allSkillSlots;

        /// <summary> </summary>
        [JsonIgnore]
        public List<int> AllSkillSlots
        {
            get
            {
                return _allSkillSlots ?? (_allSkillSlots = new List<int>
                {
                    -1,
                    1,
                    2,
                    3,
                    4,
                    5,
                    6,
                    7,
                    8
                });
            }
        }
    }
}
